// Import necessary functions from d3 library
import {
  select, // Function to select DOM elements
  csv, // Function to load CSV data
  scaleLinear, // Function to create linear scale
  scaleTime, // Function to create time scale
  extent, // Function to compute the extent of values
  axisLeft, // Function to create left-oriented axis
  axisBottom, // Function to create bottom-oriented axis
  line, // Function to create line generator
  max, // Function to compute maximum value in an array
} from 'd3';

// Select the SVG element from the HTML
const svg = select('svg');

// Get width and height attributes of the SVG element
const width = +svg.attr('width');
const height = +svg.attr('height');

// Function to render the line chart with provided data
const render = (data) => {
  // Define the title of the chart
  const title = ' Sales from 2000 to 2013';

  // Define a function to extract 'Year' data for x-axis
  const xValue = (d) => d.Year; // Display dates on x axis
  const xAxisLabel = 'Year'; // Label for x-axis
  // Define a function to extract 'Expns' data for y-axis
  const yValue = (d) => d.Expns; // Display sales value on y axis
  const yAxisLabel = 'Cost'; // Label for y-axis
  // Define margins for the plot
  const margin = {
    top: 60,
    right: 40,
    bottom: 88,
    left: 150,
  };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

  // Append a group element to the SVG and center it
  const g = svg
    .append('g')
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`,
    );
  // Create X scale with time values
  const xScale = scaleTime()
    .domain([new Date(2009, 0, 1), extent(data, xValue)[1]]) // Adjusted domain to start from 2009
    .range([0, innerWidth])
    .nice();
  // Create X axis
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(15);

  // Create Y scale with sales values
  const yScale = scaleLinear()
    .domain([30000, max(data, yValue)]) // Adjusted domain to start from 35k
    .range([innerHeight, 0])
    .nice();

  // Create Y axis
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10);

  // Append Y axis to SVG
  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();
  yAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', -90)
    .attr('x', -innerHeight / 3)
    .attr('fill', 'black')
    .attr('transform', `rotate(-90)`)
    .text(yAxisLabel);

  // Append X axis to SVG
  const xAxisG = g
    .append('g')
    .call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);
  xAxisG.select('.domain').remove();
  xAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', 70)
    .attr('x', innerWidth / 2)
    .attr('fill', 'black')
    .text(xAxisLabel);
  // Create line generator for line chart
  const lineGenerator = line()
    .x((d) => xScale(xValue(d)))
    .y((d) => yScale(yValue(d))); // Coordinates in pixels
  // Append line graph to SVG
  g.append('path')
    .attr('class', 'line_path')
    .attr('d', lineGenerator(data))
    .attr('fill', 'none')
    .attr('stroke', 'steelblue');
  // Append title of graph to SVG
  g.append('text')
    .attr('class', 'title')
    .attr('y', -10)
    .text(title);
};

// Load data from URI and render the line chart
csv(
  'https://gist.githubusercontent.com/nandinimaddula/6b773d56bd45a47ef8367896a94cd4a2/raw/70ed91d14af7612ee2b3bbf6fcd11e316653c4e7/Revenue.csv',
).then((data) => {
  // Convert 'Expns' string data to numbers and 'Year' string data to Date objects
  data.forEach((d) => {
    d.Expns = +d.Expns; // Convert string to number
    d.Year = new Date(d.Year); // Convert string to Date object
  });
  render(data); // Call render function with loaded data
});
